import torch
import torch.nn as nn
from torch.autograd import Function

class LogLUFunction(Function):
    @staticmethod
    def forward(ctx, input):
        epsilon = 1e-8
        ctx.save_for_backward(input)
        # We clip input to avoid log(0) for x = 1
        input_clipped = torch.clamp(input, max=1.0 - 1e-8)
        output = torch.where(input > 0, input, -torch.log(1 - input_clipped) + epsilon)
        return output

    @staticmethod
    def backward(ctx, grad_output):
        (input,) = ctx.saved_tensors
        # Avoid division by zero
        input_clipped = torch.clamp(input, max=1.0 - 1e-8)
        grad_input = torch.where(input > 0, torch.ones_like(input), 1.0 / (1.0 - input_clipped))
        return grad_output * grad_input

class LogLU(nn.Module):
    def __init__(self):
        super(LogLU, self).__init__()

    def forward(self, input):
        return LogLUFunction.apply(input)




from setuptools import setup, find_packages

setup(
    name='loglu-pytorch',  
    version='0.0.1',
    description='LogLU Activation Function - PyTorch',
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    author= 'Murumuri Naga Poorni ',
    author_email='poorni.m0405@gmail.com',
    packages=find_packages(),
    url='https://github.com/XenReZ/LogLU-PyTorch',
    license="Apache-2.0",
    install_requires=[ 
        "torch>=1.6.0"
    ],
    classifiers=[
    'Programming Language :: Python :: 3',
    "License :: OSI Approved :: Apache Software License",
    'Operating System :: OS Independent',
    'Intended Audience :: Developers',
    'Intended Audience :: Science/Research', 
    'Topic :: Scientific/Engineering :: Artificial Intelligence',
    ],
    python_requires='>=3.6',
)
